module Aggkit

  module Exec

    def error(message)
      STDERR.puts "Error: #{message}"
    end

    def die(message, code: 1)
      error(message)
      exit!(code)
    end

    def capture(cmd)
      puts "Capturing: #{cmd}"
      io = IO.popen(cmd)
      io.read.strip
    ensure
      io.close rescue nil
    end

    def capture!(cmd, error = nil)
      output = capture(cmd)
      error = error ? "#{error}: Execution failed" : 'Execution failed'
      die(error) unless ($?.success? rescue nil)

      output
    rescue Exception => e
      error = error ? "#{error}: #{e.inspect}" : "Execution failed: #{e.inspect}"
      die(error)
    end

    def execute!(cmd, error = nil)
      puts "Executing: #{cmd}"
      error = error ? "#{error}: Execution failed" : 'Execution failed'
      die(error) unless system(cmd)
    end

    extend ::Aggkit::Exec

  end

end

